Package: apcluster
Type: Package
Title: Affinity Propagation Clustering
Version: 1.4.14
Date: 2025-09-09
Depends: R (>= 3.3.0)
Imports: Rcpp (>= 0.11.1), methods, Matrix, stats, graphics, grDevices
Suggests: knitr
Authors@R: c(person(given = "Ulrich",
                    family = "Bodenhofer",
                    role = c("aut", "cre"),
                    email = "ulrich@bodenhofer.com"),
             person(given = "Johannes",
                    family = "Palme",
                    role = "ctb"),
             person(given = "Chrats",
                    family = "Melkonian",
                    role = "ctb"),
             person(given = "Andreas",
                    family = "Kothmeier",
                    role = "aut"),
             person(given = "Nikola",
                    family = "Kostic",
                    role = "ctb"))
Maintainer: Ulrich Bodenhofer <ulrich@bodenhofer.com>
Description: Implements Affinity Propagation clustering introduced by Frey and
	Dueck (2007) <DOI:10.1126/science.1136800>. The algorithms are largely
        analogous to the 'Matlab' code published by Frey and Dueck.
        The package further provides leveraged affinity propagation and an
        algorithm for exemplar-based agglomerative clustering that can also be
        used to join clusters obtained from affinity propagation. Various
        plotting functions are available for analyzing clustering results.
License: GPL (>= 2)
Collate: AllClasses.R AllGenerics.R access-methods.R coerce-methods.R
        show-methods.R labels-methods.R length-methods.R revDend.R
        heatmap-methods.R plot-methods.R cutree-methods.R
        sort-methods.R aggExCluster-methods.R apcluster-methods.R
        apclusterL-methods.R apclusterK-methods.R apclusterDemo.R
        preferenceRange-methods.R similarity.R simpleDist.R
        conversions.R
URL: https://github.com/UBod/apcluster
VignetteBuilder: knitr
LinkingTo: Rcpp
LazyLoad: yes
Repository: CRAN
NeedsCompilation: yes
Packaged: 2025-09-09 10:59:47 UTC; ulrich
Author: Ulrich Bodenhofer [aut, cre],
  Johannes Palme [ctb],
  Chrats Melkonian [ctb],
  Andreas Kothmeier [aut],
  Nikola Kostic [ctb]
Date/Publication: 2025-09-22 17:48:40 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:38:54 UTC; unix
Archs: apcluster.so.dSYM
